SystemOrganization addCategory: #'AlgebraicDataType-Tests'!
SystemOrganization addCategory: #AlgebraicDataType!
SystemOrganization addCategory: #'AlgebraicDataType-Trees'!

Object subclass: #AlgebraicDataType
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType'!

!AlgebraicDataType commentStamp: 'fbs 5/25/2011 07:30' prior: 0!
My subclasses represent algebraic types. My subclasses know how to destructure themselves through implementing #unapply.

Further, for unification between two graphs of instances of my subclasses, you may think of the instances as functions. For instance, Node left: a right: b is conceptually the same as some function Node(left, right) taking Trees as parameters. Thus, you may access a structure's parts through a parameter-like API in #paramAt:, #numArgs, etc.!

!AlgebraicDataType methodsFor: 'comparing' stamp: 'fbs 5/22/2011 21:14'!
= anObject
	^ (anObject respondsTo: #unapply)
		ifTrue: [self unapply = anObject unapply]
		ifFalse: [anObject isCollection
			ifTrue: [self unapply = anObject]
			ifFalse: [false]]! !

!AlgebraicDataType methodsFor: 'testing' stamp: 'fbs 5/27/2011 22:06'!
arity
	^ self children size.! !

!AlgebraicDataType methodsFor: 'accessing' stamp: 'fbs 5/24/2011 22:06'!
children
	^ self unapply allButFirst.! !

!AlgebraicDataType methodsFor: 'testing' stamp: 'fbs 5/30/2011 13:28'!
hasChildren
	"Return true if self contains substructures/is of the form f(x1, x2, ..., xn)."
	"This covers most bases, but see MetaVariable for an exception."
	^ self children isEmpty not.! !

!AlgebraicDataType methodsFor: 'comparing' stamp: 'fbs 5/25/2011 22:54'!
hash
	"17's a prime."
	^ self unapply inject: 17 into: [:acc :each | 17 * each hash].! !

!AlgebraicDataType methodsFor: 'testing' stamp: 'fbs 5/25/2011 22:34'!
isAlgebraic
	^ true.! !

!AlgebraicDataType methodsFor: 'accessing' stamp: 'fbs 5/25/2011 23:06'!
type
	^ self unapply first.! !

!AlgebraicDataType methodsFor: 'destructuring' stamp: 'fbs 5/22/2011 20:29'!
unapply
	self subclassResponsibility.! !

AlgebraicDataType subclass: #Tree
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Trees'!

Tree class
	instanceVariableNames: 'EmptyTree'!

Tree class
	instanceVariableNames: 'EmptyTree'!

Tree subclass: #Empty
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Trees'!

!Empty methodsFor: 'destructuring' stamp: 'fbs 5/22/2011 21:05'!
unapply
	^ #(#Empty).! !

Tree subclass: #Leaf
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Trees'!

!Leaf class methodsFor: 'instance creation' stamp: 'fbs 5/22/2011 20:19'!
value: anObject
	^ self new value: anObject.! !

!Leaf methodsFor: 'destructuring' stamp: 'fbs 5/22/2011 21:16'!
unapply
	^ {#Leaf. value.}! !

!Leaf methodsFor: 'accessing' stamp: 'fbs 5/22/2011 20:20'!
value
	^ value.! !

!Leaf methodsFor: 'private' stamp: 'fbs 5/22/2011 20:19'!
value: anObject
	value := anObject.! !

Tree subclass: #Node
	instanceVariableNames: 'left right'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Trees'!

!Node class methodsFor: 'instance creation' stamp: 'fbs 5/22/2011 20:21'!
left: aLeftObject 
	^ self left: aLeftObject right: Tree empty.! !

!Node class methodsFor: 'instance creation' stamp: 'fbs 5/22/2011 20:21'!
left: aLeftObject right: aRightObject
	^ self new left: aLeftObject right: aRightObject.! !

!Node class methodsFor: 'instance creation' stamp: 'fbs 5/22/2011 20:21'!
right: aRightObject
	^ self left: Tree empty right: aRightObject.! !

!Node methodsFor: 'accessing' stamp: 'fbs 5/22/2011 20:22'!
left
	^ left.! !

!Node methodsFor: 'private' stamp: 'fbs 5/22/2011 20:22'!
left: aLeftObject right: aRightObject
	left := aLeftObject.
	right := aRightObject.! !

!Node methodsFor: 'accessing' stamp: 'fbs 5/22/2011 20:22'!
right
	^ right.! !

!Node methodsFor: 'destructuring' stamp: 'fbs 5/22/2011 20:51'!
unapply
	^ {#Node. left. right.}! !

!Tree class methodsFor: 'accessing' stamp: 'fbs 5/22/2011 20:27'!
empty
	^ EmptyTree.! !

!Tree class methodsFor: 'class initialization' stamp: 'fbs 5/22/2011 20:28'!
initialize
	EmptyTree := Empty new.! !

!Object methodsFor: '*AlgebraicDataType' stamp: 'fbs 5/25/2011 22:34'!
isAlgebraic
	^ false.! !

TestCase subclass: #EmptyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Tests'!

!EmptyTest methodsFor: 'testing' stamp: 'fbs 5/25/2011 07:38'!
testEmptyTreeHasNoChildren
	self deny: Empty new hasChildren.
	self deny: Tree empty hasChildren.
	
	self assert: Tree empty children isEmpty.! !

!EmptyTest methodsFor: 'testing' stamp: 'fbs 5/25/2011 22:33'!
testEmptyTreeIsNotVariable
	self deny: Empty new isMetaVariable.
	self deny: Tree empty isMetaVariable.! !

!EmptyTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 21:09'!
testEqualsTestsStructuralEquality
	| t1 t2 t3 |
	t1 := Empty new.
	t2 := Empty new.
	t3 := t1 unapply.
	self deny: t1 == t2. "Not the same object."
	self assert: t1 = t2. "But are equal objects."
	self assert: t1 = t3. "Equality means structural equality."! !

!EmptyTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 21:27'!
testUnapplyShouldDestructure
	self assert: Tree empty unapply = {#Empty}.! !

TestCase subclass: #LeafTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Tests'!

!LeafTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 21:15'!
testEqualsTestsStructuralEquality
	| t1 t2 t3 |
	t1 := Leaf value: 1.
	t2 := Leaf value: 1.
	t3 := t1 unapply.
	self deny: t1 == t2. "Not the same object."
	self assert: t1 = t2. "But are equal objects."
	self assert: t1 = t3. "Equality means structural equality."! !

!LeafTest methodsFor: 'testing' stamp: 'fbs 5/24/2011 22:07'!
testLeafHasChildren
	self assert: (Leaf value: 1) hasChildren.
	
	self assert: (Leaf value: 1) children = #(1).! !

!LeafTest methodsFor: 'testing' stamp: 'fbs 5/25/2011 22:32'!
testLeafIsNotVariable
	self deny: (Leaf value: 1) isMetaVariable.! !

!LeafTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 21:15'!
testUnapplyShouldDestructure
	| destructure tree |
	tree := Leaf value: 1.
	destructure := tree unapply.
	self assert: destructure first = #Leaf.
	self assert: destructure second = tree value.! !

TestCase subclass: #NodeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AlgebraicDataType-Tests'!

!NodeTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 20:58'!
testEqualsTestsStructuralEquality
	| t1 t2 t3 |
	t1 := Node left: Tree empty.
	t2 := Node left: Tree empty.
	t3 := t1 unapply.
	self deny: t1 == t2. "Not the same object."
	self assert: t1 = t2. "But are equal objects."
	self assert: t1 = t3. "Equality means structural equality."! !

!NodeTest methodsFor: 'testing' stamp: 'fbs 5/24/2011 22:08'!
testNodeHasChildren
	self assert: (Node left: Tree empty right: Tree empty) hasChildren.
	
	self assert: (Node left: (Leaf value: 1) right: (Leaf value: 2)) children = {Leaf value: 1. Leaf value: 2}.! !

!NodeTest methodsFor: 'testing' stamp: 'fbs 5/25/2011 22:32'!
testNodeIsNotVariable
	self deny: (Node left: Tree empty right: Tree empty) isMetaVariable.! !

!NodeTest methodsFor: 'testing' stamp: 'fbs 5/22/2011 20:59'!
testUnapplyShouldDestructureLazily
	| destructure tree |
	tree := Node left: (Leaf value: 1) right: (Leaf value: 2).
	destructure := tree unapply.
	self assert: destructure first = #Node.
	self assert: destructure second = tree left.
	self assert: destructure third = tree right.! !
Tree initialize!
